package org.okiproject.serviceapi.authorization;
/*

   Copyright (c) 2001 Massachusetts Institute of Technology

   Permission is hereby granted, free of charge, to any
   person obtaining a copy of this software and associated
   documentation files (the "Software"), to deal in the
   Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the
   Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice
   shall be included in all copies or substantial portions of
   the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
   KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE.

*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceapi/authorization/Factory.java,v $
 */
/**
<P>
<H2>Factory</H2>
<P>
The Factory allows the application developer to create Authorization
objects in the Authorization database, to get Authorization objects from the Authorization
database given selection criteris, to ask questions of Authorization such as
 who can do what with whom.
<P>
The key objects in Authorization are Authorizations, Functions, People, and
and Qualifiers.  There are some other objects in Roles that play
supporting roles.  They are FunctionCatagory, QualifierType, Field,
and FieldValue.
<P>
There are two methods to create Authorizations. One uses method uses
Person, Function, and Qualifier.  The other adds effective date and
expiration date.  For the method without the dates the effective date
is today and there is no expiration date.
<P>
There are methods for getting Authorizations:
<pre>
   (1) getAuthsSimpleForm

       This method is overloaded to allow for combinations
       of filtering arguments to find matching Authorizations.

   (2) getAuthsForDoingFWithQ

       Given a Function and a Qualifier, finds a list of all
       Authorizations that would permit a person to do the
       Function with the Qualifier. This method is overloaded
       so that Authorizations for Functions within a FunctionCategory
       can be found. This uses a different algorithm than
       getAuthsSimpleForm.

       Note that the Qualifier in any found Authorization
       might be a parent of the Qualifier give as an argument.

   (3) getAuthsGeneralForm

       Give a set of criteria/value paris, this method finds
       all matching Authorizations. With the general form
       the application developer can do anything that the
       other two forms of getAuths can do and a lot more.
</pre>
         * @see org.okiproject.serviceapi.authorization.Authorization
         * @see org.okiproject.serviceapi.authorization.Authorizations
         * @see org.okiproject.serviceapi.authorization.Person
         * @see org.okiproject.serviceapi.common.Agents
         * @see org.okiproject.serviceapi.authorization.Function
         * @see org.okiproject.serviceapi.authorization.Functions
         * @see org.okiproject.serviceapi.authorization.FunctionCategory
         * @see org.okiproject.serviceapi.authorization.Qualifier
         * @see org.okiproject.serviceapi.authorization.Qualifiers
         * @see org.okiproject.serviceapi.authorization.QualifierType
         * @see org.okiproject.serviceapi.authorization.Field
         * @see org.okiproject.serviceapi.authorization.FieldValue
         * @see org.okiproject.serviceapi.authorization.GrantPrivilege

<p>
 * <p>
 * See the {@link org.okiproject.License License}.
 *
 * @version $Revision: 1.2 $ / $Date: 2002/02/22 16:03:51 $
*/

public interface Factory
extends org.okiproject.serviceapi.common.Factory {

  /**
   * Method getDatabaseSymbolicNames
   *
   *
   * @return String[]
   *
   * @throws Exception
   *
   */
  String[] getDatabaseSymbolicNames()
  throws Exception;

  /**
   * Method getDatabaseSymbolicName
   *
   *
   * @return String
   *
   * @throws Exception
   *
   */
  String getDatabaseSymbolicName()
  throws Exception;

  /**
   * Method setDatabaseSymbolicName
   *
   *
   * @param databaseSymbolicName
   *
   * @throws Exception
   *
   */
  void setDatabaseSymbolicName(String databaseSymbolicName)
  throws Exception;

  /**
   * Method createAuthorization
   *
   * Creates a new Authorization in the Authorizations DB for a Person
   * doing a Function with a Qualifier and having Grant rights
   * and ability to doFunction.
   *
   * @param person
   * @param function
   * @param qualifier
   * @param grant
   * @param doFunction
   * @param effectiveDate
   * @param expirationDate
   *
   * @return org.okiproject.serviceapi.authorization.Authorization
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorization createAuthorization(
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.Function function,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          org.okiproject.serviceapi.authorization.GrantPrivilege grant,
          Boolean doFunction, java.sql.Timestamp effectiveDate,
          java.sql.Timestamp expirationDate)
  throws Exception;

  /**
   * Method createAuthorization
   *
   * Creates a new Authorization in the Roles DB for a Person
   * doing a Function with a Qualifier and having Grant rights
   * and ability to doFunction. Uses current date/time as the
   * effectiveDate and doesn't set an exiration date.
   *
   * @param person
   * @param function
   * @param qualifier
   * @param grant
   * @param doFunction
   *
   * @return org.okiproject.serviceapi.authorization.Authorization
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorization createAuthorization(
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.Function function,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          org.okiproject.serviceapi.authorization.GrantPrivilege grant,
          Boolean doFunction)
  throws Exception;

  /**
   * Method deleteAuthorization
   *
   * Deletes an existing authorization in the Authorizations DB.
   *
   * @param authorization
   *
   * @throws Exception
   *
   */
  void deleteAuthorization(
          org.okiproject.serviceapi.authorization.Authorization authorization)
  throws Exception;

  /**
   * Method updateAuthorization
   *
   * Updates one or more fields in an existing authorization
   * in the Authorizations DB.
   *
   * @param authorization
   * @param person
   * @param function
   * @param qualifier
   * @param grant
   * @param doFunction
   * @param effectiveDate
   * @param expirationDate
   *
   * @throws Exception
   *
   */
  void updateAuthorization(
          org.okiproject.serviceapi.authorization.Authorization authorization,
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.Function function,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          org.okiproject.serviceapi.authorization.GrantPrivilege grant,
          Boolean doFunction, java.sql.Timestamp effectiveDate,
          java.sql.Timestamp expirationDate)
  throws Exception;

  /**
   * Method getWhoCanDo
   *
   * Given a Function and a Qualifier
   * returns an enumeration of all people allowed
   * to do the Function with the Qualifier.
   *
   * @param function
   * @param qualifier
   * @param isActiveNow
   *
   * @return org.okiproject.serviceapi.common.Agents
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.common.Agents getWhoCanDo(
          org.okiproject.serviceapi.authorization.Function function,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow)
  throws Exception;

  /**
   * Method getWhoCanDo
   *
   * Given a Function
   * returns an enumeration of all people at least one
   * Authorization for the Function where the Qualifier
   * is not considered.
   *
   * @param function
   * @param isActiveNow
   *
   * @return org.okiproject.serviceapi.common.Agents
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.common.Agents getWhoCanDo(
          org.okiproject.serviceapi.authorization.Function function,
          Boolean isActiveNow)
  throws Exception;

  /**
   * Method getWhoCanDo
   *
   * Given a FunctionCategory
   * returns an enumeration of all people with at least one
   * Authorization for at least one Function in this
   * FunctionCategory where the Qualifier
   * is not considered.
   *
   * @param functionCategory
   * @param isActiveNow
   *
   * @return org.okiproject.serviceapi.common.Agents
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.common.Agents getWhoCanDo(
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory,
          Boolean isActiveNow)
  throws Exception;

  /**
   * Method getWhoCanDo
   *
   * Given a FunctionCategory and a Qualifier
   * returns an enumeration of all people with at least one
   * Authorization for at least one Function in this
   * FunctionCategory for the Qualifier.
   *
   * @param functionCategory
   * @param qualifier
   * @param isActiveNow
   *
   * @return org.okiproject.serviceapi.common.Agents
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.common.Agents getWhoCanDo(
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow)
  throws Exception;

  /**
   * Method getWhoCanDo
   *
   * Given a Qualifier
   * returns an enumeration of all people with at least one
   * Authorization for at least one Function with the Qualifier.
   *
   * @param qualifier
   * @param isActiveNow
   *
   * @return org.okiproject.serviceapi.common.Agents
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.common.Agents getWhoCanDo(
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow)
  throws Exception;

  /**
   * Method getAuthsForDoingFWithQ
   *
   * Given a Function and a Qualifier
   * returns an enumeration of all Authorizations that would allow people
   * to do the Function with Qualifier. This method differs from the simple
   * form of getAuthsSimpleForm in that this method looks for any Authorization
   * that permits doing the Function with the Qualifier even if the Authorization's
   * Qualifier happens to be a parent of this Qualifier argument.
   *
   * @param function
   * @param qualifier
   * @param isActiveNow
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsForDoingFWithQ(
          org.okiproject.serviceapi.authorization.Function function,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow)
  throws Exception;

  /**
   * Method getAuthsForDoingFWithQ
   *
   * Given a FunctionCategory and a Qualifier
   * returns an enumeration of all Authorizations that would allow people
   * to do Functions in the FunctionCategory with Qualifier. This method differs
   * from getAuthsSimpleForm in that this method looks for any Authorization
   * that permits doing the Function with the Qualifier even if the
   * Authorization's Qualifier happens to be a parent of the Qualifier argument.
   *
   * @param functionCategory
   * @param qualifier
   * @param isActiveNow
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsForDoingFWithQ(
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow)
  throws Exception;

  /**
   * Method getAuthsSimpleForm
   *
   * Given at a Person, a Function, and a Qualifier
   * returns an enumeration of matching authorizations.
   *
   * @param person
   * @param function
   * @param qualifier
   * @param isActiveNow
   * @param expandFunctions
   * @param expandQualifiers
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsSimpleForm(
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.Function function,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow, Boolean expandFunctions,
          Boolean expandQualifiers)
  throws Exception;

  /**
   * Method getAuthsSimpleForm
   *
   * Given at a Person, a FunctionCategory, and a Qualifier
   * returns an enumeration of matching authorizations.
   * The authorizations must be for Functions within the
   * given FunctionCategory.
   *
   * @param person
   * @param functionCategory
   * @param qualifier
   * @param isActiveNow
   * @param expandFunctions
   * @param expandQualifiers
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsSimpleForm(
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow, Boolean expandFunctions,
          Boolean expandQualifiers)
  throws Exception;

  /**
   * Method getAuthsSimpleForm
   *
   * Given a Person and a Function
   * returns an enumeration of Authorizations
   * matching the Person and the Function where
   * Qualifier is not considered.
   *
   * @param person
   * @param function
   * @param isActiveNow
   * @param expandFunctions
   * @param expandQualifiers
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsSimpleForm(
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.Function function,
          Boolean isActiveNow, Boolean expandFunctions,
          Boolean expandQualifiers)
  throws Exception;

  /**
   * Method getAuthsSimpleForm
   *
   * Given a Person and a FunctionCategory
   * returns an enumeration of Authorizations
   * matching the Person and any Function in
   * the given FunctionCategory where Qualifier
   * is not considered.
   *
   * @param person
   * @param functionCategory
   * @param isActiveNow
   * @param expandFunctions
   * @param expandQualifiers
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsSimpleForm(
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory,
          Boolean isActiveNow, Boolean expandFunctions,
          Boolean expandQualifiers)
  throws Exception;

  /**
   * Method getAuthsSimpleForm
   *
   * Given a Person and a Qualifier
   * returns an enumeration of Authorizations
   * matching Person and Qualifier where
   * Function is not considered.
   *
   * @param person
   * @param qualifier
   * @param isActiveNow
   * @param expandFunctions
   * @param expandQualifiers
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsSimpleForm(
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow, Boolean expandFunctions,
          Boolean expandQualifiers)
  throws Exception;

  /**
   * Method getAuthsSimpleForm
   *
   * Given a Person
   * returns an enumeration of Authorizations
   * matching the Person where Function and
   * Qualifier are not considered.
   *
   * @param person
   * @param isActiveNow
   * @param expandFunctions
   * @param expandQualifiers
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsSimpleForm(
          org.okiproject.serviceapi.authorization.Person person, Boolean isActiveNow,
          Boolean expandFunctions, Boolean expandQualifiers)
  throws Exception;

  /**
   * Method getAuthsSimpleForm
   *
   * Given a Function and a Qualifier
   * returns an enumeration of Authorizations
   * matching the Function and the Qualifier
   * where Person is not considered.
   *
   * @param function
   * @param qualifier
   * @param isActiveNow
   * @param expandFunctions
   * @param expandQualifiers
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsSimpleForm(
          org.okiproject.serviceapi.authorization.Function function,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow, Boolean expandFunctions,
          Boolean expandQualifiers)
  throws Exception;

  /**
   * Method getAuthsSimpleForm
   *
   * Given a Qualifier
   * returns an enumeration of Authorizations
   * matching the given Qualifier where Function
   * and Person are not considered.
   *
   * @param qualifier
   * @param isActiveNow
   * @param expandFunctions
   * @param expandQualifiers
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsSimpleForm(
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow, Boolean expandFunctions,
          Boolean expandQualifiers)
  throws Exception;

  /**
   * Method getAuthsSimpleForm
   *
   * Given a FunctionCategory and a Qualifier
   * returns an enumeration of Authorizations
   * matching Functions of the given FunctionCategory
   * and Qualifier where Person is not considered.
   *
   * @param functionCategory
   * @param qualifier
   * @param isActiveNow
   * @param expandFunctions
   * @param expandQualifiers
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsSimpleForm(
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory,
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          Boolean isActiveNow, Boolean expandFunctions,
          Boolean expandQualifiers)
  throws Exception;

  /**
   * Method getAuthsSimpleForm
   *
   * Given a Function
   * returns an enumeration of Authorizations
   * matching the given Function where Person
   * and Qualifier are not considered.
   *
   * @param function
   * @param isActiveNow
   * @param expandFunctions
   * @param expandQualifiers
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsSimpleForm(
          org.okiproject.serviceapi.authorization.Function function,
          Boolean isActiveNow, Boolean expandFunctions,
          Boolean expandQualifiers)
  throws Exception;

  /**
   * Method getAuthsGeneralForm
   *
   *
   * @param selectionCriteria
   *
   * @return org.okiproject.serviceapi.authorization.Authorizations
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Authorizations getAuthsGeneralForm(
          org.okiproject.serviceapi.authorization.FieldValue[] selectionCriteria)
  throws Exception;

  /**
   * Method getMinimumAuthorizationFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getMinimumAuthorizationFields()
  throws Exception;

  /**
   * Method getDefaultAuthorizationFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getDefaultAuthorizationFields()
  throws Exception;

  /**
   * Method getCompleteAuthorizationFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getCompleteAuthorizationFields()
  throws Exception;

  /**
   * Method getCurrentAuthorizationFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getCurrentAuthorizationFields()
  throws Exception;

  /**
   * Method setCurrentAuthorizationFields
   *
   *
   * @param currentFields
   *
   * @throws Exception
   *
   */
  void setCurrentAuthorizationFields(
          org.okiproject.serviceapi.authorization.Field[] currentFields)
  throws Exception;

  /**
   * Method isAuthorized
   *
   * Given a Person, Function, and Qualifier
   * returns true if the Person is authorized to perform
   * the given Function with the given Qualifier.
   *
   * @param person
   * @param function
   * @param qualifier
   *
   * @return Boolean
   *
   * @throws Exception
   *
   */
  Boolean isAuthorized(
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.Function function,
          org.okiproject.serviceapi.authorization.Qualifier qualifier)
  throws Exception;

  /**
   * Method isAuthorized
   *
   * Given a Person and Qualifier
   * returns true if the Person is authorized to perform
   * at least one Function with the given Qualifier.
   *
   * @param person
   * @param qualifier
   *
   * @return Boolean
   *
   * @throws Exception
   *
   */
  Boolean isAuthorized(
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.Qualifier qualifier)
  throws Exception;

  /**
   * Method isAuthorized
   *
   * Given a Person, FunctionCategory, and Qualifier
   * returns true if the Person is authorized to perform
   * at least one Function in the FunctionCategory with the Qualifier.
   *
   * @param person
   * @param functionCategory
   * @param qualifier
   *
   * @return Boolean
   *
   * @throws Exception
   *
   */
  Boolean isAuthorized(
          org.okiproject.serviceapi.authorization.Person person,
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory,
          org.okiproject.serviceapi.authorization.Qualifier qualifier)
  throws Exception;

  /**
   * Method createFunction
   *
   *
   * @param name
   * @param description
   * @param functionCategory
   * @param qualifierType
   *
   * @return org.okiproject.serviceapi.authorization.Function
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Function createFunction(String name,
          String description,
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory,
          org.okiproject.serviceapi.authorization.QualifierType qualifierType)
  throws Exception;

  /**
   * Method deleteFunction
   *
   *
   * @param function
   *
   * @throws Exception
   *
   */
  void deleteFunction(
          org.okiproject.serviceapi.authorization.Function function)
  throws Exception;

  /**
   * Method isFunctionFoundInAuthorizations
   *
   *
   * @param function
   *
   * @return Boolean
   *
   * @throws Exception
   *
   */
  Boolean isFunctionFoundInAuthorizations(
          org.okiproject.serviceapi.authorization.Function function)
  throws Exception;

  /**
   * Method refreshFunction
   *
   * Refreshs Function properties from a changing Authorizations DB.
   *
   * @param function
   *
   * @throws Exception
   *
   */
  void refreshFunction(
          org.okiproject.serviceapi.authorization.Function function)
  throws Exception;

  /**
   * Method updateFunction
   *
   * Updates one or more fields in an existing function in the Authorizations DB.
   *
   * @param function
   * @param newName
   * @param newDescription
   * @param newFunctionCategory
   * @param newQualifierType
   *
   * @throws Exception
   *
   */
  void updateFunction(
          org.okiproject.serviceapi.authorization.Function function, String newName,
          String newDescription,
          org.okiproject.serviceapi.authorization.FunctionCategory newFunctionCategory,
          org.okiproject.serviceapi.authorization.QualifierType newQualifierType)
  throws Exception;

  /**
   * Method createFunctionCategory
   *
   *
   * @param category
   * @param description
   *
   * @return org.okiproject.serviceapi.authorization.FunctionCategory
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.FunctionCategory createFunctionCategory(
          String category, String description)
  throws Exception;

  /**
   * Method deleteFunctionCategory
   *
   *
   * @param functionCategory
   *
   * @throws Exception
   *
   */
  void deleteFunctionCategory(
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory)
  throws Exception;

  /**
   * Method isFunctionCategoryFoundInAuthorizations
   *
   *
   * @param functionCategory
   *
   * @return Boolean
   *
   * @throws Exception
   *
   */
  Boolean isFunctionCategoryFoundInAuthorizations(
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory)
  throws Exception;

  /**
   * Method refreshFunctionCategory
   *
   *
   * @param functionCategory
   *
   * @throws Exception
   *
   */
  void refreshFunctionCategory(
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory)
  throws Exception;

  /**
   * Method updateFunctionCategory
   *
   *
   * @param functionCategory
   * @param newDescription
   *
   * @throws Exception
   *
   */
  void updateFunctionCategory(
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory,
          String newDescription)
  throws Exception;

  /**
   * Method getFunctionCategories
   *
   *
   * @return org.okiproject.serviceapi.authorization.FunctionCategory[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.FunctionCategory[] getFunctionCategories()
  throws Exception;

  /**
   * Method getFunctionsInCategory
   *
   *
   * @param functionCategory
   *
   * @return org.okiproject.serviceapi.authorization.Functions
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Functions getFunctionsInCategory(
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory)
  throws Exception;

  /**
   * Method getFunctionsGeneralForm
   *
   *
   * @param selectionCriteria
   *
   * @return org.okiproject.serviceapi.authorization.Functions
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Functions getFunctionsGeneralForm(
          org.okiproject.serviceapi.authorization.FieldValue[] selectionCriteria)
  throws Exception;

  /**
   * Method getMinimumFunctionFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getMinimumFunctionFields()
  throws Exception;

  /**
   * Method getDefaultFunctionFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getDefaultFunctionFields()
  throws Exception;

  /**
   * Method getCompleteFunctionFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getCompleteFunctionFields()
  throws Exception;

  /**
   * Method getCurrentFunctionFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getCurrentFunctionFields()
  throws Exception;

  /**
   * Method setCurrentFunctionFields
   *
   *
   * @param currentFields
   *
   * @throws Exception
   *
   */
  void setCurrentFunctionFields(
          org.okiproject.serviceapi.authorization.Field[] currentFields)
  throws Exception;

  /**
   * Method newFunction
   *
   * Returns new instance of Function. The instance is local to the application.
   * It may or may not exist in the Authorizations DB.
   *
   * @param functionCategory
   * @param name
   *
   * @return org.okiproject.serviceapi.authorization.Function
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Function newFunction(
          org.okiproject.serviceapi.authorization.FunctionCategory functionCategory,
          String name)
  throws Exception;

  /**
   * Method newFunction
   *
   * Returns a new instance of Function. The instance is local to the application.
   * It may or may not exist in the Authorizationss DB. This method will create an instance
   * of org.okiproject.serviceapi.authorization.Function from the string
   * functionCategory.
   *
   * @param functionCategory
   * @param name
   *
   * @return org.okiproject.serviceapi.authorization.Function
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Function newFunction(
          String functionCategory, String name)
  throws Exception;

  /**
   * Method newFunctionCategory
   *
   * Returns a new instance of FunctionCategory. The instance is local to the application.
   * It may or may not exist in the Authorizations DB.
   *
   * @param functionCategory
   *
   * @return org.okiproject.serviceapi.authorization.FunctionCategory
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.FunctionCategory newFunctionCategory(
          String functionCategory)
  throws Exception;

  /**
   * Method newGrantPrivilege
   *
   * The instance created is local to the application.
   * It may or may not exist in the Authorizations DB.
   *
   * <pre>
   * Values:
   *    org.okiproject.serviceapi.authorization.GrantPrivilege.NOTGRANTABLE
   *    org.okiproject.serviceapi.authorization.GrantPrivilege.GRANTABLE
   *    org.okiproject.serviceapi.authorization.GrantPrivilege.GRANTABLEFORGRANTING
   * </pre>
   *
   * @param grantPrivilege
   *
   * @return org.okiproject.serviceapi.authorization.GrantPrivilege
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.GrantPrivilege newGrantPrivilege(
          String grantPrivilege)
  throws Exception;

  /**
   * Method getPeopleGeneralForm
   *
   *
   * @param selectionCriteria
   *
   * @return org.okiproject.serviceapi.common.Agents
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.common.Agents getPeopleGeneralForm(
          org.okiproject.serviceapi.authorization.FieldValue[] selectionCriteria)
  throws Exception;

  /**
   * Method getMinimumPersonFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getMinimumPersonFields()
  throws Exception;

  /**
   * Method getDefaultPersonFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getDefaultPersonFields()
  throws Exception;

  /**
   * Method getCompletePersonFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getCompletePersonFields()
  throws Exception;

  /**
   * Method getCurrentPersonFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getCurrentPersonFields()
  throws Exception;

  /**
   * Method setCurrentPersonFields
   *
   *
   * @param currentFields
   *
   * @throws Exception
   *
   */
  void setCurrentPersonFields(
          org.okiproject.serviceapi.authorization.Field[] currentFields)
  throws Exception;

  /**
   * Method newPerson
   *
   * Returns a new instance of Person. The instance is local to the application.
   * The person may or may not exist in the Authorizations DB.
   *
   * @param key
   *
   * @return org.okiproject.serviceapi.authorization.Person
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Person newPerson(Object key)
  throws Exception;

  /**
   * Method isPersonFoundInAuthorizations
   *
   *
   * @param person
   *
   * @return Boolean
   *
   * @throws Exception
   *
   */
  Boolean isPersonFoundInAuthorizations(
          org.okiproject.serviceapi.authorization.Person person)
  throws Exception;


  /**
   * Method refreshPerson
   *
   * Refresh properties from a changing Authorizations DB.
   *
   * @param person
   *
   * @throws Exception
   *
   */
  void refreshPerson(org.okiproject.serviceapi.authorization.Person person)
  throws Exception;


  /**
   * Method createQualifier
   *
   * Creates a new Qualifier in the Authorizations DB. This is different
   * from making a new instance of a Qualifier object locally
   * as the Qualifier will be inserted into the Authorizations DB.
   *
   * @param qualifierType
   * @param qualifierCode
   * @param qualifierDescription
   * @param parent
   *
   * @return org.okiproject.serviceapi.authorization.Qualifier
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Qualifier createQualifier(
          org.okiproject.serviceapi.authorization.QualifierType qualifierType,
          String qualifierCode, String qualifierDescription,
          org.okiproject.serviceapi.authorization.Qualifier parent)
  throws Exception;

  /**
   * Method deleteQualifier
   *
   *
   * @param qualifier
   *
   * @throws Exception
   *
   */
  void deleteQualifier(
          org.okiproject.serviceapi.authorization.Qualifier qualifier)
  throws Exception;

  /**
   * Method isQualifierFoundInRoles
   *
   * Determines if this Qualifier has a matching Qualifier in the Authorizations DB.
   *
   * @param qualifier
   *
   * @return Boolean
   *
   * @throws Exception
   *
   */
  Boolean isQualifierFoundInRoles(
          org.okiproject.serviceapi.authorization.Qualifier qualifier)
  throws Exception;

  /**
   * Method refreshQualifier
   *
   * Refreshes properties from a changing Authorizations DB.
   *
   * @param qualifier
   *
   * @throws Exception
   *
   */
  void refreshQualifier(
          org.okiproject.serviceapi.authorization.Qualifier qualifier)
  throws Exception;

  /**
   * Method updateQualifier
   *
   * Updates one or more fields in an existing qualifier in the Authorizations DB.
   *
   * @param qualifier
   * @param newQualifierCode
   * @param newDescription
   *
   * @throws Exception
   *
   */
  void updateQualifier(
          org.okiproject.serviceapi.authorization.Qualifier qualifier,
          String newQualifierCode, String newDescription)
  throws Exception;

  /**
   * Method createQualifierType
   *
   * Creates a new QualifierType in the Authorizations DB. This is different
   * from making a new instance of a QualifierType object locally
   * as the QualifierType will be inserted into the Authorizations DB.
   *
   * @param type
   * @param description
   *
   * @return org.okiproject.serviceapi.authorization.QualifierType
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.QualifierType createQualifierType(
          String type, String description)
  throws Exception;

  /**
   * Method deleteQualifierType
   *
   *
   * @param qualifierType
   *
   * @throws Exception
   *
   */
  void deleteQualifierType(
          org.okiproject.serviceapi.authorization.QualifierType qualifierType)
  throws Exception;

  /**
   * Method isQualifierTypeFoundInRoles
   *
   * Determines if this QualifierType has a matching QualifierType in the Authorizations DB.
   *
   * @param qualifierType
   *
   * @return Boolean
   *
   * @throws Exception
   *
   */
  Boolean isQualifierTypeFoundInRoles(
          org.okiproject.serviceapi.authorization.QualifierType qualifierType)
  throws Exception;

  /**
   * Method refreshQualifierType
   *
   * Refreshes QualifierType properties from a changing Authorizations DB.
   *
   * @param qualifierType
   *
   * @throws Exception
   *
   */
  void refreshQualifierType(
          org.okiproject.serviceapi.authorization.QualifierType qualifierType)
  throws Exception;

  /**
   * Method updateQualifierType
   *
   * Updates Description in an existing qualifierType in the Authorizations DB.
   *
   * @param qualifierType
   * @param newDescription
   *
   * @throws Exception
   *
   */
  void updateQualifierType(
          org.okiproject.serviceapi.authorization.QualifierType qualifierType,
          String newDescription)
  throws Exception;

  /**
   * Method getQualifierTypes
   *
   * Returns Qualifier types available in the Authorizations DB.
   *
   * @return org.okiproject.serviceapi.authorization.QualifierType[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.QualifierType[] getQualifierTypes()
  throws Exception;

  /**
   * Method getRootQualifier
   *
   * Given a Qualifier Type
   * returns the qualifier that is the root of the tree of qualifiers of this type.
   *
   * @param qualifierType
   *
   * @return org.okiproject.serviceapi.authorization.Qualifier
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Qualifier getRootQualifier(
          org.okiproject.serviceapi.authorization.QualifierType qualifierType)
  throws Exception;

  /**
   * Method getQualifierChildren
   *
   * Given an existing qualifier
   * returns a list of its child qualifiers.
   *
   * @param qualifier
   *
   * @return org.okiproject.serviceapi.authorization.Qualifiers
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Qualifiers getQualifierChildren(
          org.okiproject.serviceapi.authorization.Qualifier qualifier)
  throws Exception;

  /**
   * Method getQualifiersGeneralForm
   *
   * Given a list of selection criteria field/value pairs
   * returns a list of Qualifiers.
   *
   * @param selectionCriteria
   *
   * @return org.okiproject.serviceapi.authorization.Qualifiers
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Qualifiers getQualifiersGeneralForm(
          org.okiproject.serviceapi.authorization.FieldValue[] selectionCriteria)
  throws Exception;

  /**
   * Method getMinimumQualifierFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getMinimumQualifierFields()
  throws Exception;

  /**
   * Method getDefaultQualifierFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getDefaultQualifierFields()
  throws Exception;

  /**
   * Method getCompleteQualifierFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getCompleteQualifierFields()
  throws Exception;

  /**
   * Method getCurrentQualifierFields
   *
   *
   * @return org.okiproject.serviceapi.authorization.Field[]
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Field[] getCurrentQualifierFields()
  throws Exception;

  /**
   * Method setCurrentQualifierFields
   *
   *
   * @param currentFields
   *
   * @throws Exception
   *
   */
  void setCurrentQualifierFields(
          org.okiproject.serviceapi.authorization.Field[] currentFields)
  throws Exception;

  /**
   * Method newQualifier
   *
   * Returns a new instance of org.okiproject.serviceapi.authorization.Qualifier.
   * The instance may or may not exist in the Authorizations DB. This is a local
   * instance.
   *
   * @param qualifierType
   * @param qualifierCode
   *
   * @return org.okiproject.serviceapi.authorization.Qualifier
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Qualifier newQualifier(
          org.okiproject.serviceapi.authorization.QualifierType qualifierType,
          String qualifierCode)
  throws Exception;

  /**
   * Method newQualifier
   *
   * Returns a new instance of org.okiproject.serviceapi.authorization.Qualifier.
   * The instance may or may not exist in the Authorizations DB. This is a local
   * instance.
   *
   * @param qualifierType
   * @param qualifierCode
   *
   * @return org.okiproject.serviceapi.authorization.Qualifier
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.Qualifier newQualifier(
          String qualifierType, String qualifierCode)
  throws Exception;

  /**
   * Method newQualifierType
   *
   * Returns a new instance of org.okiproject.serviceapi.authorization.QualifierType.
   * The instance may or may not exist in the Authorizations DB. This is a local
   * instance.
   *
   * @param qualifierType
   *
   * @return org.okiproject.serviceapi.authorization.QualifierType
   *
   * @throws Exception
   *
   */
  org.okiproject.serviceapi.authorization.QualifierType newQualifierType(
          String qualifierType)
  throws Exception;

}
