package org.okiproject.serviceapi.file;
/*
   Copyright (c) 2001 Massachusetts Institute of Technology

   Permission is hereby granted, free of charge, to any
   person obtaining a copy of this software and associated
   documentation files (the "Software"), to deal in the
   Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the
   Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice
   shall be included in all copies or substantial portions of
   the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
   KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE.
*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceapi/file/ByteStore.java,v $ 
 */
/**
Key interface in file package
<p>
The ByteStore is the key interface.  It is a set of bytes with associated
physical metadata.
<p>
If the Cabinet implementation supports it, each ByteStore may be
stored redundantly on multiple servers for protection against data
loss.
<p>
If the Cabinet implementation supports versioning, it may have several
versions of arbitrary depth.
<p>
For ease of use, the ByteStore interface parallels when appropriate the
java.io.File class.
<p>
Versioning may be supported in the Cabinet.  The default is to return
the latest version of a ByteStore.  Other versions may be accessed by a
key string.  Each version is itself a ByteStore, and may have versions
of its own, permitting branching.  The underlying principle is that
all versions should serve the same basic purpose, have the same
mime-type, be functionally equivalent.  If idiosyncrasies of a
particular version are significant, that version key may be
specified.  When a version key is specified, the base version rather
than any latest sub-version is returned.  Sub versions may then be
obtained if desired.  Different versions will only differ in bytes of
content and therefore length, lastModified time, and digest/checksum.
A particular version may be given a GUID only by creating a new
ByteStore in the Cabinet.
<hr>
<p>
@version $Revision: 1.9 $ / $Date: 2002/02/16 05:02:53 $
*/
public interface ByteStore
extends java.io.Serializable {

  // from java.io.File

  /**
   * Method length
   *
   *
   * @return long
   *
   */
  public long length();

  /**
   * Method lastModified
   *
   *
   * @return long
   *
   */
  public long lastModified();

  /**
   * Method setLastModified
   *
   *
   * @param time
   *
   * @return boolean
   *
   */
  public boolean setLastModified(long time);

  /**
   * Method deleteOnExit
   *
   *
   */
  public void deleteOnExit();

  /**
   * Method toString
   *
   *
   * @return String
   *
   */
  public String toString();        // guid?

  /**
   * Method canRead
   *
   *
   * @return boolean
   *
   */
  public boolean canRead();

  /**
   * Method canWrite
   *
   *
   * @return boolean
   *
   */
  public boolean canWrite();

  /**
   * Method setReadOnly
   *
   *
   * @return boolean
   *
   */
  public boolean setReadOnly();

  /**
   * Method setWritable
   *
   *
   * @return boolean
   *
   */
  public boolean setWritable();    // not in File, but seems needed?

  /**
   * Method compareTo
   *
   *
   * @param o
   *
   * @return int
   *
   */
  public int compareTo(Object o);

  /**
   * Method equals
   *
   *
   * @param obj
   *
   * @return boolean
   *
   */
  public boolean equals(Object obj);

  /**
   * Method hashCode
   *
   *
   * @return int
   *
   */
  public int hashCode();

  // added for OKI

  /**
   * Method getGUID
   *
   *
   * @return org.okiproject.serviceapi.guid.GUID
   *
   */
  public org.okiproject.serviceapi.guid.GUID getGUID();

  /**
   * Method getByteStoreName
   *
   *
   * @return String
   *
   */
  public String getByteStoreName();

  /**
   * Method renameTo
   *
   *
   * @param newName
   *
   * @return boolean
   *
   */
  public boolean renameTo(String newName);

  /**
   * Method getMimeType
   *
   *
   * @return java.lang.String
   *
   */
  public java.lang.String getMimeType();    // Content-Type in jar file manifest

  /**
   * Method setMimeType
   *
   *
   * @param mimeType
   *
   */
  public void setMimeType(java.lang.String mimeType);

  // e.g. md5 or crc

  /**
   * Method getDigest
   *
   *
   * @param digestAlgorithm
   *
   * @return java.lang.String
   *
   */
  public java.lang.String getDigest(String digestAlgorithm);

  // manifest attributes

  /**
   * Method getAttributes
   *
   *
   * @return java.util.jar.Attributes
   *
   */
  public java.util.jar.Attributes getAttributes();

  // primarily for quota

  /**
   * Method getOwner
   *
   *
   * @return org.okiproject.serviceapi.common.Agent
   *
   */
  public org.okiproject.serviceapi.common.Agent getOwner();

  /**
   * Method setOwner
   *
   *
   * @param owner
   *
   */
  public void setOwner(org.okiproject.serviceapi.common.Agent owner);

  // for ease of use for i/o

  /**
   * Method getInputStream
   *
   *
   * @return java.io.InputStream
   *
   */
  public java.io.InputStream getInputStream();

  /**
   * Method getOutputStream
   *
   *
   * @return java.io.OutputStream
   *
   */
  public java.io.OutputStream getOutputStream();

  /**
   * Method getReader
   *
   *
   * @return java.io.Reader
   *
   */
  public java.io.Reader getReader();

  /**
   * Method getWriter
   *
   *
   * @return java.io.Writer
   *
   */
  public java.io.Writer getWriter();

  // built on basic read() and write() methods

  // versioning:

  /**
   * Method hasVersions
   *
   *
   * @return boolean
   *
   */
  public boolean hasVersions();

  /**
   * Method getLatestVersion
   *
   *
   * @return ByteStore
   *
   */
  public ByteStore getLatestVersion();

  /**
   * Method getBaseVersion
   *
   *
   * @return ByteStore
   *
   */
  public ByteStore getBaseVersion();

  /**
   * Method getVersion
   *
   *
   * @param versionKey
   *
   * @return ByteStore
   *
   */
  public ByteStore getVersion(java.lang.String versionKey);

  /**
   * Method getVersions
   *
   *
   * @return ByteStore[]
   *
   */
  public ByteStore[] getVersions();

  /**
   * Method newVersion
   *
   *
   * @param agent
   * @param versionKey
   *
   * @return ByteStore
   *
   */
  public ByteStore newVersion(org.okiproject.serviceapi.common.Agent agent,
                              String versionKey);
}

