package org.okiproject.serviceapi.logging;

/*
Copyright (c) 2001 Massachusetts Institute of Technology

Permission is hereby granted, free of charge, to any
person obtaining a copy of this software and associated
documentation files (the "Software"), to deal in the
Software without restriction, including without limitation
the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the
Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice
shall be included in all copies or substantial portions of
the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
IN THE SOFTWARE.
*/

/*
 * $Source: /cvs/oki/org/okiproject/serviceapi/logging/Log.java,v $
 */

/**
<p>
Purpose: Represents the logging strategy used, allowing replaceable
         logging strategies.
<p>
AKA:  Log Handler
<p>
@version $Revision: 1.6 $ / $Date: 2002/03/07 22:23:24 $
*/
public interface Log
extends java.io.Serializable {

  /** To get the appication specific name  */
  public String getCategory();

  /** To set the appication specific name  */

  /* Return a locally unique Identifier ? */
  public void setCategory(final String cat);

  /** To open the log */
  public void start(final org.okiproject.serviceapi.common.Agent who);

  /** To close the log */
  public void stop(final org.okiproject.serviceapi.common.Agent who);

  /** Workhorse used to append to  the log */
  public void appendLog(

  //final org.okiproject.serviceapi.common.Agent who,
  final org.okiproject.serviceapi.logging.LogLevel priority, final String message);

  /** This is needed to tell the individual logs what the base priority
                        of the logging service is.

			@param who Represents the Agent state for authn/authz purposes.
			@param priority LogLevel representing the logging service's base priority.
  */
  public void setBasePriority(
    final org.okiproject.serviceapi.common.Agent who,
    final org.okiproject.serviceapi.logging.LogLevel priority);

  /**
	 *      Method readLog
	 *
	 *      An interface to read the log. It is assumed that
   *      any semantics on the format on the log will be set by
   *      the application, which will also know how to parse and
   *      reformat the messages for reporting or viewing purposes,
   *      and handle presentation aspects as it sees fit.
   *
   *     Only a low-level access to the contents is provided via this interface.
   *
   *      @param  who Represents the Agent state for authn/authz purposes.
   *      @param  b               Destination array.
   *      @param  offset
   *      @param  count           # of bytes to read.
   *
   *
   *      @return int
   *      @throws Exception   if any I/O or other error occurs. Semantic errors
   *  include trying to read handles that cannot be read from, such as stderr,
   *  stdout.
   */
  public int readLog(final org.okiproject.serviceapi.common.Agent who,
                     final byte[] b, final int offset, final int count)
  throws Exception;

  /**
	 *    Method readLog
	 *
   *    Another interface to read the log. This would return an enumeration
   *    of all messages matching a certain query string.
   *
   *    @param  who Represents the Agent state for authn/authz purposes.
   *    @param  queryString Represents tag(s) to look for in the message
   *
   *    @return java.util.Iterator
   *    @throws Exception       if any I/O or other error occurs.
   */
  public java.util.Iterator readLog(
    final org.okiproject.serviceapi.common.Agent who,
    final String queryString)
  throws Exception;

  /** JavaBean patterns */

  /**
   *            Method setFormatSpecification
   *
   *            @param who
   *            @param org.okiproject.serviceapi.logging.FormatSpecification
   *
   */
  public void setFormatSpecification(
    final org.okiproject.serviceapi.common.Agent who,
    final org.okiproject.serviceapi.logging.FormatSpecification format);

  /**
   * Method getFormatSpecification
   *
   * @param who
   *
   * @return org.okiproject.serviceapi.logging.FormatSpecification
   *
   */
  public org.okiproject.serviceapi.logging.FormatSpecification getFormatSpecification(
    final org.okiproject.serviceapi.common.Agent who);
}
