package org.okiproject.serviceimpl.mit.guid;
/*

   Copyright (c) 2001 Massachusetts Institute of Technology

   Permission is hereby granted, free of charge, to any
   person obtaining a copy of this software and associated
   documentation files (the "Software"), to deal in the
   Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the
   Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice
   shall be included in all copies or substantial portions of
   the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
   KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE.

*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceimpl/mit/guid/MitGUID.java,v $ 
 */
/**
<p>
************************************************************************
*	Purpose: Implementation of a simple guid generator using a database.
* $Source: /cvs/oki/org/okiproject/serviceimpl/mit/guid/MitGUID.java,v $
* @version $Revision: 1.11 $
* $Date: 2002/02/16 05:02:54 $
*	@author		<B>Giri Swaminathan, MIT</B>
**********************************************************************
 * <p>
 * See the {@link org.okiproject.License License}.
 *
 * @version $Revision: 1.11 $ / $Date: 2002/02/16 05:02:54 $
*/
//import org.okiproject.serviceapi.sql.*;
//import org.okiproject.serviceimpl.mit.sql.*;
public class MitGUID implements org.okiproject.serviceapi.guid.GUID,
java.io.Serializable
{
	private long id = 0;
	private final String procSQL = "{? = call GET_NEXT_INSTANCE (?)}";
	private final String addSQL = "{? = call ADD_GUID_TYPE (?)}";
	private String propFile = null;
	private String guidType = null;
	private java.sql.Connection c = null;
	
	private static final void Usage() {
		System.out.println("Usage: MitGUID <propsFile> <guidtype>");
	}

	public static void main( final String[] args) {
		int ac = args.length;
		if ( ac < 2 ) {
			Usage();
			System.exit( 1 );
		}
		try {	
			final String propsFile = args[ 0 ];
			final String guidType = args[ 1 ];
			MitGUID mitGuidFactory = new MitGUID( propsFile, guidType);
			// getInterfaceInfo( mitGuidFactory );
			Object guid = mitGuidFactory.getGUID( null, guidType );
		  System.out.println( "Got guid = " + guid  + " for guidType " +
				 guidType);
		} catch( java.sql.SQLException ex) {
				System.out.println("\n--- SQLException caught ---\n");
				while (ex != null) {
						System.out.println("Message:   " + ex.getMessage ());
						System.out.println("SQLState:  " + ex.getSQLState ());
						System.out.println("ErrorCode: " + ex.getErrorCode ());
						ex = ex.getNextException();
						System.out.println();
				}
		} catch ( final Exception genex ) {
			genex.printStackTrace();
		}
	}

	public MitGUID( final String propFile, final String guidType ) throws Exception {
		this.propFile = propFile;
		this.guidType = guidType;
	}
		 
/**
 *	This will create a GUID for a given kind of OKI entity
	@param	Agent 	Caller ID
	@param	String 	prefix or "type" of GUID

	@return	Object representing opaque handle.
*/
	public Object getGUID( final org.okiproject.serviceapi.common.Agent agent, 
		final String prefix ) throws org.okiproject.serviceapi.guid.GUIDException {
				// if agent 
				// return getIdForType( prefix );
					return execGUIDfunc( procSQL, prefix );
				//return getIdForType( prefix );
		}

/**
 *	An administrative interface (presumably authenticated) to add new GUID
 *	types in the system.
 *	@param	Agent 	Caller ID
 *	@param	String 	prefix or "type" of GUID
 *
 */
	public void addType( final org.okiproject.serviceapi.common.Agent agent,
		final String type ) throws org.okiproject.serviceapi.guid.GUIDException {
			/* Ignores return value */
				execGUIDfunc( addSQL, type );
	}

 //public String[] getTypes(Agent a);

	private final Long execGUIDfunc( final String SQL, final String type ) 
		throws org.okiproject.serviceapi.guid.GUIDException {

		java.sql.Connection c = null;
		java.sql.CallableStatement cs = null;

		try {
			c = new org.okiproject.serviceimpl.mit.guid.IDBMSConnectionFactoryImpl().getConnection( propFile );
			cs = c.prepareCall( SQL );
			cs.registerOutParameter (1, java.sql.Types.INTEGER);
			cs.setString(2, type);
			cs.execute();
			int id = cs.getInt( 1 );
			return new Long( id );
		} catch ( java.sql.SQLException ex ) {
				while (ex != null) {
						System.out.println("Message:   " + ex.getMessage ());
						System.out.println("SQLState:  " + ex.getSQLState ());
						System.out.println("ErrorCode: " + ex.getErrorCode ());
						ex = ex.getNextException();
						System.out.println();
				}
				throw new org.okiproject.serviceapi.guid.GUIDException( 
																								ex.getLocalizedMessage() );
		} catch ( final Exception ex2 ) {
				ex2.printStackTrace();
				throw new org.okiproject.serviceapi.guid.GUIDException( 
						ex2.getLocalizedMessage() );
		} finally {
				try {
					cs.close();
					c.close();
				} catch ( Exception ignored ) {
					// don't care
				} finally {
					cs = null;
					c = null;
				}
			}
	}

	private final Long getIdForType( final String type ) 
		throws org.okiproject.serviceapi.guid.GUIDException {

		java.sql.Connection c = null;
		java.sql.CallableStatement cs = null;

		try {
			c = new org.okiproject.serviceimpl.mit.guid.IDBMSConnectionFactoryImpl().getConnection( propFile );
			cs = c.prepareCall( procSQL );
			cs.registerOutParameter (1, java.sql.Types.INTEGER);
			cs.setString(2, type);
			cs.execute();
			int id = cs.getInt( 1 );
			return new Long( id );
		} catch ( java.sql.SQLException ex ) {
				while (ex != null) {
						System.out.println("Message:   " + ex.getMessage ());
						System.out.println("SQLState:  " + ex.getSQLState ());
						System.out.println("ErrorCode: " + ex.getErrorCode ());
						ex = ex.getNextException();
						System.out.println();
				}
				throw new org.okiproject.serviceapi.guid.GUIDException( 
																								ex.getLocalizedMessage() );
		} catch ( final Exception ex2 ) {
				ex2.printStackTrace();
				throw new org.okiproject.serviceapi.guid.GUIDException( 
						ex2.getLocalizedMessage() );
		} finally {
				try {
					cs.close();
					c.close();
				} catch ( Exception ignored ) {
					// don't care
				} finally {
					cs = null;
					c = null;
				}
			}
	}

}
