package org.okiproject.serviceimpl.mit.sql ;
/*

   Copyright (c) 2001 Massachusetts Institute of Technology

   Permission is hereby granted, free of charge, to any
   person obtaining a copy of this software and associated
   documentation files (the "Software"), to deal in the
   Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the
   Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice
   shall be included in all copies or substantial portions of
   the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
   KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
   WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
   PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
   DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
   CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
   IN THE SOFTWARE.

*/
/*
 * $Source: /cvs/oki/org/okiproject/serviceimpl/mit/sql/Connection.java,v $ 
 */
/**
<p>
 * <p>
 * See the {@link org.okiproject.License License}.
 *
 * @version $Revision: 1.3 $ / $Date: 2002/02/16 17:10:52 $
*/
public class Connection
implements org.okiproject.serviceapi.dbc.Connection
{
	private static final String EXECUTEUPDATE = "executeUpdate" ;
	private static final String CALLPROCEDURE = "callProcedure" ;
	private static final String COLUMN_NAME = "COLUMN_NAME" ;
	private static final String COLUMN_TYPE = "COLUMN_TYPE" ;
	private static final String DATA_TYPE = "DATA_TYPE" ;
	private static final String SEQUENCE = "SEQUENCE" ;

	public org.okiproject.serviceapi.dbc.Table getProcedures
		( String catalog
		, String schemaPattern
		, String procedureNamePattern
		)
		throws Exception
	{
		return org.okiproject.serviceimpl.mit.sql.Table.getProcedures
			( this
			, catalog
			, schemaPattern
			, procedureNamePattern
			) ;
	}

	public org.okiproject.serviceapi.dbc.Table getProcedureColumns
		( String catalog
		, String schemaPattern
		, String procedureName
		, String columnNamePattern
		)
		throws Exception
	{
		return org.okiproject.serviceimpl.mit.sql.Table.getProcedureColumns
			( this
			, catalog
			, schemaPattern
			, procedureName
			, columnNamePattern
			) ;
	}

	public org.okiproject.serviceapi.dbc.Table getTables()
		throws Exception
	{
		return org.okiproject.serviceimpl.mit.sql.Table.getTables
			( this
			) ;
	}

	public org.okiproject.serviceapi.dbc.Table getTables
		( String catalog
		, String schemaPattern
		, String tableNamePattern
		, String[] tableTypes
		)
		throws Exception
	{
		return org.okiproject.serviceimpl.mit.sql.Table.getTables
			( this
			, catalog
			, schemaPattern
			, tableNamePattern
			, tableTypes
			) ;
	}

	public org.okiproject.serviceapi.dbc.Table executeQuery( String sql )
		throws Exception
	{
		return org.okiproject.serviceimpl.mit.sql.Table.executeQuery
			( this
			, sql
			) ;
	}

	public org.okiproject.serviceapi.dbc.Table executeQuery
		( String sql
		, Object[] objs
		)
		throws Exception
	{
		return org.okiproject.serviceimpl.mit.sql.Table.executeQuery
			( this
			, sql
			, objs
			) ;
	}

	public org.okiproject.serviceapi.dbc.Table executeQuery
		( String sql
		, Object[] objs
		, int[] targetSqlTypes
		)
		throws Exception
	{
		return org.okiproject.serviceimpl.mit.sql.Table.executeQuery
			( this
			, sql
			, objs
			, targetSqlTypes
			) ;
	}

	public org.okiproject.serviceapi.dbc.Table executeQuery
		( String sql
		, Object[] objs
		, int[] targetSqlTypes
		, int[] scales
		)
		throws Exception
	{
		return org.okiproject.serviceimpl.mit.sql.Table.executeQuery
			( this
			, sql
			, objs
			, targetSqlTypes
			, scales
			) ;
	}

	public int executeUpdate( String sql )
		throws Exception
	{
		if( hasClient() )
		{
			return executeUpdateRemotely( sql ) ;
		}
		return executeUpdateLocally( sql ) ;
	}

	private int executeUpdateRemotely( String sql )
		throws Exception
	{
		Class[] argsTypes = new Class[ 1 ] ;
		Object[] args = new Object[ 1 ] ;
		argsTypes[ 0 ] = sql.getClass() ;
		args[ 0 ] = sql ;
		Number num = ( Number ) getClient().invoke( this , EXECUTEUPDATE , argsTypes , args ) ;
		return num.intValue() ;
	}

	public boolean hasClient()
		throws Exception
	{
		return org.okiproject.serviceimpl.mit.util.RuntimeContext.contains
			( new String( clientstr ) ) ;
	}

	public org.okiproject.serviceimpl.mit.rmissl.Client getClient()
		throws Exception
	{
		if( hasClient() )
		{
			return ( org.okiproject.serviceimpl.mit.rmissl.Client ) org.okiproject.serviceimpl.mit.util.RuntimeContext.getProperty
					( new String( clientstr ) ) ;
		}
		return null ;
	}

	public int executeUpdateLocally( String sql )
		throws Exception
	{
		java.sql.Statement statement = null ;
		try
		{
			java.sql.Connection conn = getConnection() ;
			statement = conn.createStatement() ;
			int rowCount = statement.executeUpdate( sql ) ;
			statement.close() ; statement = null ;
			return rowCount ;
		}
		catch( Exception ex )
		{
			if( null != statement )
			{
				statement.close() ;
			}
			throw ex ;
		}
	}

	public int executeUpdate( String sql , Object[] objs )
		throws Exception
	{
		if( hasClient() )
		{
			return executeUpdateRemotely( sql , objs ) ;
		}
		return executeUpdateLocally( sql , objs ) ;
	}

	private int executeUpdateRemotely( String sql , Object[] objs )
		throws Exception
	{
		Class[] argsTypes = new Class[ 2 ] ;
		Object[] args = new Object[ 2 ] ;
		argsTypes[ 0 ] = sql.getClass() ;
		argsTypes[ 1 ] = objs.getClass() ;
		args[ 0 ] = sql ;
		args[ 1 ] = objs ;
		Number num = ( Number ) getClient().invoke( this , EXECUTEUPDATE , argsTypes , args ) ;
		return num.intValue() ;
	}

	public int executeUpdateLocally( String sql , Object[] objs )
		throws Exception
	{
		java.sql.PreparedStatement statement = null ;
		try
		{
			java.sql.Connection conn = getConnection() ;
			statement = conn.prepareStatement( sql ) ;
			for( int i = 0 ; objs.length != i ; i ++ )
			{
				statement.setObject( i + 1 , objs[ i ] ) ;
			}
			int rowCount = statement.executeUpdate() ;
			statement.close() ; statement = null ;
			return rowCount ;
		}
		catch( Exception ex )
		{
			if( null != statement )
			{
				statement.close() ;
			}
			throw ex ;
		}
	}

	public int executeUpdate( String sql , Object[] objs , int[] targetSqlTypes )
		throws Exception
	{
		if( hasClient() )
		{
			return executeUpdateRemotely( sql , objs , targetSqlTypes ) ;
		}
		return executeUpdateLocally( sql , objs  , targetSqlTypes ) ;
	}

	private int executeUpdateRemotely
		( String sql
		, Object[] objs
		, int[] targetSqlTypes
		)
		throws Exception
	{
		Class[] argsTypes = new Class[ 3 ] ;
		Object[] args = new Object[ 3 ] ;
		argsTypes[ 0 ] = sql.getClass() ;
		argsTypes[ 1 ] = objs.getClass() ;
		argsTypes[ 2 ] = targetSqlTypes.getClass() ;
		args[ 0 ] = sql ;
		args[ 1 ] = objs ;
		args[ 2 ] = targetSqlTypes ;
		Number num = ( Number ) getClient().invoke( this , EXECUTEUPDATE , argsTypes , args ) ;
		return num.intValue() ;
	}

	public int executeUpdateLocally( String sql , Object[] objs , int[] targetSqlTypes )
		throws Exception
	{
		java.sql.PreparedStatement statement = null ;
		try
		{
			java.sql.Connection conn = getConnection() ;
			statement = conn.prepareStatement( sql ) ;
			for( int i = 0 ; objs.length != i ; i ++ )
			{
				statement.setObject( i + 1 , objs[ i ] , targetSqlTypes[ i ] ) ;
			}
			int rowCount = statement.executeUpdate() ;
			statement.close() ; statement = null ;
			return rowCount ;
		}
		catch( Exception ex )
		{
			if( null != statement )
			{
				statement.close() ;
			}
			throw ex ;
		}
	}

	public int executeUpdate( String sql , Object[] objs , int[] targetSqlTypes , int[] scales )
		throws Exception
	{
		if( hasClient() )
		{
			return executeUpdateRemotely( sql , objs , targetSqlTypes , scales ) ;
		}
		return executeUpdateLocally( sql , objs  , targetSqlTypes , scales ) ;
	}

	private int executeUpdateRemotely
		( String sql
		, Object[] objs
		, int[] targetSqlTypes
		, int[] scales
		)
		throws Exception
	{
		Class[] argsTypes = new Class[ 4 ] ;
		Object[] args = new Object[ 4 ] ;
		argsTypes[ 0 ] = sql.getClass() ;
		argsTypes[ 1 ] = objs.getClass() ;
		argsTypes[ 2 ] = targetSqlTypes.getClass() ;
		argsTypes[ 3 ] = scales.getClass() ;
		args[ 0 ] = sql ;
		args[ 1 ] = objs ;
		args[ 2 ] = targetSqlTypes ;
		args[ 3 ] = scales ;
		Number num = ( Number ) getClient().invoke( this , EXECUTEUPDATE , argsTypes , args ) ;
		return num.intValue() ;
	}

	public int executeUpdateLocally( String sql , Object[] objs , int[] targetSqlTypes , int [] scales )
		throws Exception
	{
		java.sql.PreparedStatement statement = null ;
		try
		{
			java.sql.Connection conn = getConnection() ;
			statement = conn.prepareStatement( sql ) ;
			for( int i = 0 ; objs.length != i ; i ++ )
			{
				statement.setObject( i + 1 , objs[ i ] , targetSqlTypes[ i ] , scales[ i ] ) ;
			}
			int rowCount = statement.executeUpdate() ;
			statement.close() ; statement = null ;
			return rowCount ;
		}
		catch( Exception ex )
		{
			if( null != statement )
			{
				statement.close() ;
			}
			throw ex ;
		}
	}

	public void commit()
		throws Exception
	{
		try
		{
			if( ! getAutoCommit().booleanValue() )
			{
				getConnection().commit() ;
			}
		}
		catch( Exception ex )
		{
			resetConnection() ;
			throw ex ;
		}
	}

	public void rollback()
		throws Exception
	{
		try
		{
			if( ! getAutoCommit().booleanValue() )
			{
				getConnection().rollback() ;
			}
		}
		catch( Exception ex )
		{
			resetConnection() ;
			throw ex ;
		}
	}

	public org.okiproject.serviceapi.dbc.Table callProcedure
		( String procedureName
		, Object[] args
		, int[] columnTypes
		, int[] dataTypes
		)
		throws Exception
	{
		return callProcedure
			( ( String ) null
			, ( String ) null
			, procedureName
			, ( String ) null
			, args
			, columnTypes
			, dataTypes
			) ;
	}

	public org.okiproject.serviceapi.dbc.Table callProcedure
		( String catalog
		, String schemaPattern
		, String procedureName
		, String columnNamePattern
		, Object[] args
		, int[] columnTypes
		, int[] dataTypes
		)
		throws Exception
	{
		if( hasClient() )
		{
			return callProcedureRemotely
				( catalog
				, schemaPattern
				, procedureName
				, columnNamePattern
				, args
				, columnTypes
				, dataTypes
				) ;
		}
		return callProcedureLocally
			( catalog
			, schemaPattern
			, procedureName
			, columnNamePattern
			, args
			, columnTypes
			, dataTypes
			) ;
	}

	private org.okiproject.serviceapi.dbc.Table callProcedureRemotely
		( String catalog
		, String schemaPattern
		, String procedureName
		, String columnNamePattern
		, Object[] arguments
		, int[] columnTypes
		, int[] dataTypes
		)
		throws Exception
	{
		Class[] argsTypes = new Class[ 7 ] ;
		Object[] args = new Object[ 7 ] ;
		argsTypes[ 0 ]
			= ( null != catalog ) ? catalog.getClass() : null ;
		argsTypes[ 1 ]
			= ( null != schemaPattern ) ? schemaPattern.getClass() : null ;
		argsTypes[ 2 ]
			= ( null != procedureName ) ? procedureName.getClass() : null ;
		argsTypes[ 3 ]
			= ( null != columnNamePattern ) ? columnNamePattern.getClass() : null ;
		argsTypes[ 4 ]
			= ( null != arguments ) ? arguments.getClass() : null ;
		argsTypes[ 5 ]
			= ( null != columnTypes ) ? columnTypes.getClass() : null ;
		argsTypes[ 6 ]
			= ( null != dataTypes ) ? dataTypes.getClass() : null ;
		args[ 0 ] = catalog ;
		args[ 1 ] = schemaPattern ;
		args[ 2 ] = procedureName ;
		args[ 3 ] = columnNamePattern ;
		args[ 4 ] = arguments ;
		args[ 5 ] = columnTypes ;
		args[ 6 ] = dataTypes ;
		return ( org.okiproject.serviceapi.dbc.Table ) getClient().invoke( this , CALLPROCEDURE , argsTypes , args ) ;
	}

	public org.okiproject.serviceapi.dbc.Table callProcedureLocally
		( String catalog
		, String schemaPattern
		, String procedureName
		, String columnNamePattern
		, Object[] args
		, int[] columnTypes
		, int[] dataTypes
		)
		throws Exception
	{
		java.sql.CallableStatement stmt = null ;
		try
		{
			if
			(  ( null != procedureName )
			&& ( null != args )
			&& ( null != columnTypes )
			&& ( null != dataTypes )
			)
			{
				stmt
					= setStatement
						( catalog
						, schemaPattern
						, procedureName
						, columnNamePattern
						, args
						, columnTypes
						, dataTypes
						) ;
				stmt.execute() ;
				org.okiproject.serviceapi.dbc.Table table = createTable( stmt , columnTypes , dataTypes ) ;
				stmt.close() ;
				return table ;
			}
			throw new org.okiproject.serviceapi.dbc.ConnectionException
				( org.okiproject.serviceapi.dbc.ConnectionException.INVALIDARGUMENTS ) ;
		}
		catch( Exception ex )
		{
			if( null != stmt )
			{
				stmt.close() ;
			}
			resetConnection() ;
			throw ex ;
		}
	}

	private java.sql.CallableStatement setStatement
		( String catalog
		, String schemaPattern
		, String procedureName
		, String columnNamePattern
		, Object[] args
		, int[] columnTypes
		, int[] dataTypes
		)
		throws Exception
	{
		String ret = "" ;
		String params = "" ;
		int paramCount = 0 ;
		for( int i = 0 ; columnTypes.length != i ; i ++ )
		{
			int columnType = columnTypes[ i ] ;
			switch( columnType )
			{
			case java.sql.DatabaseMetaData.procedureColumnIn:
			case java.sql.DatabaseMetaData.procedureColumnInOut:
			case java.sql.DatabaseMetaData.procedureColumnOut:
			case java.sql.DatabaseMetaData.procedureColumnResult:
				if( 0 != paramCount ) { params += ", " ; }
				params += "?" ;
				paramCount ++ ;
				break ;
			case java.sql.DatabaseMetaData.procedureColumnReturn:
				ret = "?=" ;
				break ;
			}
		}
		String sql = "{" + ret + "call " + procedureName + "(" + params + ")}" ;
		java.sql.CallableStatement stmt = getConnection().prepareCall( sql ) ;
		int index = 0 ;
		int sequence = 1 ;
		for( int i = 0 ; columnTypes.length != i ; i ++ )
		{
			int columnType = columnTypes[ i ] ;
			int datatype = dataTypes[ i ] ;
			Object arg = null ;
			if( ( null != args ) && ( args.length != index ) )
			{
				arg = args[ index ++ ] ;
			}
			if( null == arg )
			{
				arg = "" ;
			}
			switch( columnType )
			{
			case java.sql.DatabaseMetaData.procedureColumnIn:
				if( args.length != i )
				{
					if( datatype == java.sql.Types.OTHER )
					{
						stmt.setObject( sequence , arg , java.sql.Types.FLOAT , 0 ) ;
					}
					else
					{
						stmt.setObject( sequence , arg , datatype ) ;
					}
					sequence ++ ;
				}
				break ;
			case java.sql.DatabaseMetaData.procedureColumnInOut:
				if( datatype == java.sql.Types.OTHER )
				{
					stmt.setObject( sequence , arg , java.sql.Types.FLOAT , 0 ) ;
					stmt.registerOutParameter( sequence , java.sql.Types.FLOAT , 0 ) ;
				}
				else
				{
					stmt.setObject( sequence , arg , datatype ) ;
					stmt.registerOutParameter( sequence , datatype ) ;
				}
				sequence ++ ;
				break ;
			case java.sql.DatabaseMetaData.procedureColumnResult:
			case java.sql.DatabaseMetaData.procedureColumnReturn:
			case java.sql.DatabaseMetaData.procedureColumnOut:
				if( datatype == java.sql.Types.OTHER )
				{
					stmt.registerOutParameter( sequence , java.sql.Types.FLOAT , 0 ) ;
				}
				else
				{
					stmt.registerOutParameter( sequence , datatype ) ;
				}
				sequence ++ ;
				break ;
			case java.sql.DatabaseMetaData.procedureColumnUnknown:
				break ;
			}
		}
		return stmt ;
	}

	private org.okiproject.serviceapi.dbc.Table createTable
		( java.sql.CallableStatement stmt
		, int[] columnTypes
		, int[] dataTypes
		)
		throws Exception
	{
		java.util.Vector outColumnsV = new java.util.Vector() ;
		java.util.Vector outColumnsMetaDataV = new java.util.Vector() ;
		for( int i = 0 ; columnTypes.length != i ; i ++ )
		{

			switch( columnTypes[ i ] )
			{
			case java.sql.DatabaseMetaData.procedureColumnInOut:
			case java.sql.DatabaseMetaData.procedureColumnOut:
			case java.sql.DatabaseMetaData.procedureColumnReturn:
			case java.sql.DatabaseMetaData.procedureColumnResult:
				org.okiproject.serviceimpl.mit.sql.ColumnMetaData colMetaData
					= new org.okiproject.serviceimpl.mit.sql.ColumnMetaData( i + 1 , null ) ;
				colMetaData.columnType = dataTypes[ i ] ;
				outColumnsV.addElement( stmt.getObject( i + 1 ) ) ;
				outColumnsMetaDataV.addElement( colMetaData ) ;
				break ;
			default:
				break ;
			}
		}
		org.okiproject.serviceapi.dbc.ColumnMetaData[] columnsMetaData
			= new org.okiproject.serviceapi.dbc.ColumnMetaData[ outColumnsMetaDataV.size() ] ;
		Object[] columns
			= new Object[ outColumnsV.size() ] ;
		for( int i = 0 ; columns.length != i ; i ++ )
		{
			columnsMetaData[ i ]
				= ( org.okiproject.serviceapi.dbc.ColumnMetaData ) outColumnsMetaDataV.elementAt( i ) ;
			columns[ i ] = outColumnsV.elementAt( i ) ;
		}
		Object[] onerow = new Object[ 1 ] ;
		onerow[ 0 ] = ( Object ) columns ;
		org.okiproject.serviceapi.dbc.RowsMetaData rowsMetaData = new org.okiproject.serviceimpl.mit.sql.RowsMetaData( columnsMetaData ) ;
		org.okiproject.serviceapi.dbc.Rows rows2 = new org.okiproject.serviceimpl.mit.sql.Rows( rowsMetaData , onerow ) ;
		org.okiproject.serviceapi.dbc.Table table = new org.okiproject.serviceimpl.mit.sql.Table( this , rows2 ) ;
		return table ;
	}

	protected static String connstr = ".Connection" ;
	protected static String clientstr = "org.okiproject.serviceimpl.mit.rmissl.Client" ;
	protected static int ROWS_IN_BLOCK = 1000 ;

	private static final String EMPTYSTRING = "" ;
	private static final String DEFAULTCONNECTION = EMPTYSTRING ;

	private String database = EMPTYSTRING ;
	public String getDatabase() { return this.database ; }
	private void setDatabase( String database )
	{
		if( ! this.database.equals( database ) )
		{
			resetCacheId() ;
			this.database = EMPTYSTRING ;
			if( null != database )
			{
				this.database = database ;
			}
		}
	}

	private static final String DEFAULTDATABASEURLPREFIX = "jdbc:oracle:oci8:@" ;
	private String databaseURL = EMPTYSTRING ;
	public String getDatabaseURL() { return this.databaseURL ; }
	private void setDatabaseURL( String databaseURL )
	{
		if( ! this.databaseURL.equals( databaseURL ) )
		{
			resetCacheId() ;
			this.databaseURL = EMPTYSTRING ;
			if( null != databaseURL )
			{
				this.databaseURL = databaseURL ;
			}
		}
	}

	private static final String DEFAULTDRIVERNAME = "oracle.jdbc.driver.OracleDriver" ;
	private String driverName = DEFAULTDRIVERNAME ;
	public String getDriverName() { return this.driverName ; }
	public void setDriverName( String driverName ) { this.driverName = driverName ; }

	private String username = EMPTYSTRING ;
	public String getUsername() { return this.username ; }
	private void setUsername( String username )
	{
		if( ! this.username.equals( username ) )
		{
			resetCacheId() ;
			this.username = EMPTYSTRING ;
			if( null != username )
			{
				this.username = username ;
			}
		}
	}

	private String password = EMPTYSTRING ;
	private String getPassword() { return this.password ; }
	private void setPassword( String password )
	{
		if( ! this.password.equals( password ) )
		{
			resetCacheId() ;
			this.password = EMPTYSTRING ;
			if( null != password )
			{
				this.password = password ;
			}
		}
	}

	private Boolean autoCommit = new Boolean( true ) ;
	public Boolean getAutoCommit() { return this.autoCommit ; }
	public void setAutoCommit( Boolean autoCommit )
	{
		try
		{
			if( ! this.autoCommit.equals( autoCommit ) )
			{
				resetConnection() ;
				this.autoCommit = autoCommit ;
			}
		}
		catch( Exception ex )
		{
			ex.printStackTrace() ;
		}
	}

	private Connection()
	{
		super() ;
	}

	public static org.okiproject.serviceapi.dbc.Connection create
		( String databaseURL
		, String driverName
		, String username
		, String password
		, String id
		)
		throws Exception
	{
		String exStr = org.okiproject.serviceapi.dbc.ConnectionException.INVALIDUSERNAME ;
		if( null != username )
		{
			if( null != password )
			{
				if( null != databaseURL )
				{
					if( null != driverName )
					{
						org.okiproject.serviceimpl.mit.sql.Connection db = ( org.okiproject.serviceimpl.mit.sql.Connection ) new org.okiproject.serviceimpl.mit.sql.Connection() ;
						db.setDatabase( null ) ;
						db.setDatabaseURL( databaseURL ) ;
						db.setDriverName( driverName ) ;
						db.setUsername( username ) ;
						db.setPassword( password ) ;
						db.setId( id ) ;
						return db ;
					}
					else
					{
						exStr = org.okiproject.serviceapi.dbc.ConnectionException.INVALIDDRIVERNAME ;
					}
				}
				else
				{
					exStr = org.okiproject.serviceapi.dbc.ConnectionException.INVALIDDATABASEURL ;
				}
			}
			else
			{
				exStr = org.okiproject.serviceapi.dbc.ConnectionException.INVALIDPASSWORD ;
			}
		}
		throw new org.okiproject.serviceapi.dbc.ConnectionException( exStr ) ;
	}

	public static org.okiproject.serviceapi.dbc.Connection create
		( String database
		, String username
		, String password
		, String id
		)
		throws Exception
	{
		String exStr = org.okiproject.serviceapi.dbc.ConnectionException.INVALIDDATABASE ;
		if( null != database )
		{
			if( null != username )
			{
				if( null != password )
				{
					org.okiproject.serviceimpl.mit.sql.Connection db = ( org.okiproject.serviceimpl.mit.sql.Connection ) new org.okiproject.serviceimpl.mit.sql.Connection() ;
					db.setDatabase( database ) ;
					db.setDatabaseURL( null ) ;
					db.setDriverName( DEFAULTDRIVERNAME ) ;
					db.setUsername( username ) ;
					db.setPassword( password ) ;
					db.setId( id ) ;
					return db ;
				}
				else
				{
					exStr = org.okiproject.serviceapi.dbc.ConnectionException.INVALIDPASSWORD ;
				}
			}
			else
			{
				exStr = org.okiproject.serviceapi.dbc.ConnectionException.INVALIDUSERNAME ;
			}
		}
		throw new org.okiproject.serviceapi.dbc.ConnectionException( exStr ) ;
	}

	private String id = DEFAULTCONNECTION ;
	public String getId() { return this.id ; }
	private void setId( String id )
	{
		this.id = DEFAULTCONNECTION ;
		if( null != id )
		{
			this.id = id ;
		}
	}

	public synchronized boolean equals( org.okiproject.serviceapi.dbc.Connection db )
	{
		return
		(
			( this == db )
			||
			(  ( null != db )
			&& ( db instanceof org.okiproject.serviceimpl.mit.sql.Connection )
			&& ( getCacheId().equals( ( ( org.okiproject.serviceimpl.mit.sql.Connection ) db ).getCacheId() ) )
			)
		) ;
	}

	private Object cacheId = null ;
	private synchronized Object getCacheId()
	{
		if( null == this.cacheId )
		{
			Object[] objs = new Object[ 6 ] ;
			objs[ 0 ] = getDatabase() ;
			objs[ 1 ] = getDatabaseURL() ;
			objs[ 2 ] = getUsername() ;
			objs[ 3 ] = getPassword() ;
			objs[ 4 ] = getId() ;
			objs[ 5 ] = connstr ;
			this.cacheId = objs ;
		}
		return this.cacheId ;
	}
	private void resetCacheId() { this.cacheId = null ; }

	public void connect()
		throws Exception
	{
		getConnection() ;
	}

	public void disconnect()
		throws Exception
	{
		resetConnection() ;
	}

	protected java.sql.Connection getConnection()
	{
		try
		{
			java.sql.Connection conn = getConnectionFromCache() ;
			if( null == conn )
			{
				String dbURL = DEFAULTDATABASEURLPREFIX + getDatabase() ;
				if( ! EMPTYSTRING.equals( getDatabaseURL() ) )
				{
					dbURL = getDatabaseURL() ;
				}
				org.okiproject.serviceimpl.mit.sql.Connect connect
					= new org.okiproject.serviceimpl.mit.sql.Connect
						( getDriverName()
						, dbURL
						, getUsername()
						, getPassword()
						) ;
				conn = connect.getConnection() ;
				conn.setAutoCommit( getAutoCommit().booleanValue() ) ;
				org.okiproject.serviceimpl.mit.util.RuntimeContext.setProperty( getCacheId() , conn ) ;
			}
			return conn ;
		}
		catch( Exception ex )
		{
		}
		return null ;
	}

	protected java.sql.Connection getConnectionFromCache()
		throws Exception
	{
		java.util.Enumeration enum
			= org.okiproject.serviceimpl.mit.util.RuntimeContext.getKeys() ;
		if( null != enum )
		{
			while( enum.hasMoreElements() )
			{
				Object[] test = ( Object[] ) enum.nextElement() ;
				if( ( getDatabase().equals( ( String ) test[ 0 ] ) )
				&&	( getDatabaseURL().equals( ( String ) test[ 1 ] ) )
				&&	( getUsername().equals( ( String ) test[ 2 ] ) )
				&&	( getPassword().equals( ( String ) test[ 3 ] ) )
				&&	( getId().equals( ( String ) test[ 4 ] ) )
				&&	( connstr.equals( ( String ) test[ 5 ] ) )
				)
				{
					return ( java.sql.Connection )
						org.okiproject.serviceimpl.mit.util.RuntimeContext.getProperty( test ) ;
				}
			}
		}
		return null ;
	}

	protected void resetConnection()
		throws Exception
	{
		java.sql.Connection conn
			= ( java.sql.Connection ) org.okiproject.serviceimpl.mit.util.RuntimeContext.getProperty
				( getCacheId() ) ;
		if( null != conn )
		{
			conn.close() ;
			org.okiproject.serviceimpl.mit.util.RuntimeContext.setProperty( getCacheId() , null ) ;
		}
	}

	private String findDataTypeName( int datatype )
	{
		for( int i = 0 ; types.length != i ; i ++ )
		{
			if( types[ i ] == datatype )
			{
				return typeNames[ i ] ;
			}
		}
		return null ;
	}

	private static int[] types =
	{	java.sql.Types.ARRAY
	,	java.sql.Types.BIGINT
	,	java.sql.Types.BINARY
	,	java.sql.Types.BIT
	,	java.sql.Types.BLOB
	,	java.sql.Types.CHAR
	,	java.sql.Types.CLOB
	,	java.sql.Types.DATE
	,	java.sql.Types.DECIMAL
	,	java.sql.Types.DISTINCT
	,	java.sql.Types.DOUBLE
	,	java.sql.Types.FLOAT
	,	java.sql.Types.INTEGER
	,	java.sql.Types.JAVA_OBJECT
	,	java.sql.Types.LONGVARBINARY
	,	java.sql.Types.LONGVARCHAR
	,	java.sql.Types.NULL
	,	java.sql.Types.NUMERIC
	,	java.sql.Types.OTHER
	,	java.sql.Types.REAL
	,	java.sql.Types.REF
	,	java.sql.Types.SMALLINT
	,	java.sql.Types.STRUCT
	,	java.sql.Types.TIME
	,	java.sql.Types.TIMESTAMP
	,	java.sql.Types.TINYINT
	,	java.sql.Types.VARBINARY
	,	java.sql.Types.VARCHAR
	} ;

  	private static String[] typeNames =
  	{	"java.sql.Types.ARRAY"
	,	"java.sql.Types.BIGINT"
	,	"java.sql.Types.BINARY"
	,	"java.sql.Types.BIT"
	,	"java.sql.Types.BLOB"
	,	"java.sql.Types.CHAR"
	,	"java.sql.Types.CLOB"
	,	"java.sql.Types.DATE"
	,	"java.sql.Types.DECIMAL"
	,	"java.sql.Types.DISTINCT"
	,	"java.sql.Types.DOUBLE"
	,	"java.sql.Types.FLOAT"
	,	"java.sql.Types.INTEGER"
	,	"java.sql.Types.JAVA_OBJECT"
	,	"java.sql.Types.LONGVARBINARY"
	,	"java.sql.Types.LONGVARCHAR"
	,	"java.sql.Types.NULL"
	,	"java.sql.Types.NUMERIC"
	,	"java.sql.Types.OTHER"
	,	"java.sql.Types.REAL"
	,	"java.sql.Types.REF"
	,	"java.sql.Types.SMALLINT"
	,	"java.sql.Types.STRUCT"
	,	"java.sql.Types.TIME"
	,	"java.sql.Types.TIMESTAMP"
	,	"java.sql.Types.TINYINT"
	,	"java.sql.Types.VARBINARY"
	,	"java.sql.Types.VARCHAR"
	} ;

}